Package: seqHMM
Title: Mixture Hidden Markov Models for Social Sequence Data and Other
        Multivariate, Multichannel Categorical Time Series
Version: 1.2.6
Date: 2023-06-7
Authors@R: 
    c(person(given = "Jouni",
           family = "Helske",
           role = c("aut", "cre"),
           email = "jouni.helske@iki.fi",
           comment = c(ORCID = "0000-0001-7130-793X")),
      person(given = "Satu",
           family = "Helske",
           role = "aut",
           comment = c(ORCID = "0000-0003-0532-0153")))
Description: Designed for fitting hidden (latent) Markov models and mixture
    hidden Markov models for social sequence data and other categorical time series.
    Also some more restricted versions of these type of models are available: Markov
    models, mixture Markov models, and latent class models. The package supports
    models for one or multiple subjects with one or multiple parallel sequences
    (channels). External covariates can be added to explain cluster membership in
    mixture models. The package provides functions for evaluating and comparing
    models, as well as functions for visualizing of multichannel sequence data and
    hidden Markov models. Models are estimated using maximum likelihood via the EM
    algorithm and/or direct numerical maximization with analytical gradients. All
    main algorithms are written in C++ with support for parallel computation. 
    Documentation is available via several vignettes in this page, and the  
    paper by Helske and Helske (2019, <doi:10.18637/jss.v088.i03>).
LazyData: true
LinkingTo: Rcpp, RcppArmadillo
Depends: R (>= 3.5.0)
Imports: gridBase, igraph, Matrix, nloptr, numDeriv, Rcpp (>= 0.11.3),
        TraMineR (>= 1.8-8), graphics, grDevices, grid, methods, stats,
        utils
Suggests: MASS, nnet, knitr, testthat (>= 3.0.0), covr
License: GPL (>= 2)
Encoding: UTF-8
BugReports: https://github.com/helske/seqHMM/issues
VignetteBuilder: knitr
RoxygenNote: 7.2.3
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2023-07-05 10:58:56 UTC; jovetale
Author: Jouni Helske [aut, cre] (<https://orcid.org/0000-0001-7130-793X>),
  Satu Helske [aut] (<https://orcid.org/0000-0003-0532-0153>)
Maintainer: Jouni Helske <jouni.helske@iki.fi>
Repository: CRAN
Date/Publication: 2023-07-06 00:10:29 UTC
Built: R 4.4.0; x86_64-apple-darwin21.6.0; 2024-05-08 08:52:18 UTC; unix
