# Generated by roxygen2: do not edit by hand

S3method("cluster_names<-",mhmm)
S3method("state_names<-",hmm)
S3method("state_names<-",mhmm)
S3method(cluster_names,mhmm)
S3method(logLik,hmm)
S3method(logLik,mhmm)
S3method(plot,hmm)
S3method(plot,mhmm)
S3method(plot,ssp)
S3method(print,hmm)
S3method(print,mhmm)
S3method(print,summary.mhmm)
S3method(state_names,hmm)
S3method(state_names,mhmm)
S3method(summary,mhmm)
S3method(vcov,mhmm)
export("cluster_names<-")
export("state_names<-")
export(alphabet)
export(build_hmm)
export(build_lcm)
export(build_mhmm)
export(build_mm)
export(build_mmm)
export(cluster_names)
export(estimate_coef)
export(fit_hmm)
export(fit_mhmm)
export(fit_model)
export(forward_backward)
export(gridplot)
export(hidden_paths)
export(mc_to_sc)
export(mc_to_sc_data)
export(mssplot)
export(plot_colors)
export(posterior_probs)
export(separate_mhmm)
export(seqdef)
export(seqstatf)
export(simulate_emission_probs)
export(simulate_hmm)
export(simulate_initial_probs)
export(simulate_mhmm)
export(simulate_transition_probs)
export(ssp)
export(ssplot)
export(state_names)
export(trim_hmm)
export(trim_model)
import(grid)
import(gridBase)
import(igraph)
import(nloptr)
importFrom(Matrix,.bdiag)
importFrom(Rcpp,evalCpp)
importFrom(TraMineR,alphabet)
importFrom(TraMineR,is.stslist)
importFrom(TraMineR,seqdef)
importFrom(TraMineR,seqdist)
importFrom(TraMineR,seqdistmc)
importFrom(TraMineR,seqlegend)
importFrom(TraMineR,seqlength)
importFrom(TraMineR,seqplot)
importFrom(TraMineR,seqstatf)
importFrom(grDevices,col2rgb)
importFrom(grDevices,rainbow)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,polygon)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(methods,hasArg)
importFrom(numDeriv,jacobian)
importFrom(stats,BIC)
importFrom(stats,cmdscale)
importFrom(stats,complete.cases)
importFrom(stats,logLik)
importFrom(stats,model.matrix)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,vcov)
importFrom(utils,menu)
useDynLib(seqHMM, .registration = TRUE)
